@@ -1,3 +1,4 @@
+from zoneinfo import ZoneInfoNotFoundError
 from configobj import ConfigObj, ParseError
 from pgspecial.namedqueries import NamedQueries
 from .config import skip_initial_comment
@@ -19,10 +20,15 @@ from time import time, sleep
 from typing import Optional
 
 from cli_helpers.tabular_output import TabularOutputFormatter
-from cli_helpers.tabular_output.preprocessors import align_decimals, format_numbers
+from cli_helpers.tabular_output.preprocessors import (
+    align_decimals,
+    format_numbers,
+    format_timestamps,
+)
 from cli_helpers.utils import strip_ansi
 from .explain_output_formatter import ExplainOutputFormatter
 import click
+import tzlocal
 
 try:
     import setproctitle
@@ -111,12 +117,13 @@ MetaQuery.__new__.__defaults__ = ("", False, 0, 0, False, False, False, False)
 
 OutputSettings = namedtuple(
     "OutputSettings",
-    "table_format dcmlfmt floatfmt missingval expanded max_width case_function style_output max_field_width",
+    "table_format dcmlfmt floatfmt column_date_formats missingval expanded max_width case_function style_output max_field_width",
 )
 OutputSettings.__new__.__defaults__ = (
     None,
     None,
     None,
+    None,
     "<null>",
     False,
     None,
@@ -264,6 +271,7 @@ class PGCli:
         self.on_error = c["main"]["on_error"].upper()
         self.decimal_format = c["data_formats"]["decimal"]
         self.float_format = c["data_formats"]["float"]
+        self.column_date_formats = c["column_date_formats"]
         auth.keyring_initialize(c["main"].as_bool("keyring"), logger=self.logger)
         self.show_bottom_toolbar = c["main"].as_bool("show_bottom_toolbar")
 
@@ -1179,6 +1187,7 @@ class PGCli:
                 table_format=self.table_format,
                 dcmlfmt=self.decimal_format,
                 floatfmt=self.float_format,
+                column_date_formats=self.column_date_formats,
                 missingval=self.null_string,
                 expanded=expanded,
                 max_width=max_width,
@@ -1593,9 +1602,9 @@ def cli(
     if list_databases or ping_database:
         database = "postgres"
 
+    cfg = load_config(pgclirc, config_full_path)
     if dsn != "":
         try:
-            cfg = load_config(pgclirc, config_full_path)
             dsn_config = cfg["alias_dsn"][dsn]
         except KeyError:
             click.secho(
@@ -1624,6 +1633,55 @@ def cli(
     else:
         pgcli.connect(database, host, user, port)
 
+    if "use_local_timezone" not in cfg["main"] or cfg["main"].as_bool(
+        "use_local_timezone"
+    ):
+        server_tz = pgcli.pgexecute.get_timezone()
+
+        def echo_error(msg: str):
+            click.secho(
+                "Failed to determine the local time zone",
+                err=True,
+                fg="yellow",
+            )
+            click.secho(
+                msg,
+                err=True,
+                fg="yellow",
+            )
+            click.secho(
+                f"Continuing with the default time zone as preset by the server ({server_tz})",
+                err=True,
+                fg="yellow",
+            )
+            click.secho(
+                "Set `use_local_timezone = False` in the config to avoid trying to override the server time zone\n",
+                err=True,
+                dim=True,
+            )
+
+        local_tz = None
+        try:
+            local_tz = tzlocal.get_localzone_name()
+
+            if local_tz is None:
+                echo_error("No local time zone configuration found\n")
+            else:
+                click.secho(
+                    f"Using local time zone {local_tz} (server uses {server_tz})",
+                    fg="green",
+                )
+                click.secho(
+                    "Use `set time zone <TZ>` to override, or set `use_local_timezone = False` in the config",
+                    dim=True,
+                )
+
+                pgcli.pgexecute.set_timezone(local_tz)
+        except ZoneInfoNotFoundError as e:
+            # e.args[0] is the pre-formatted message which includes a list
+            # of conflicting sources
+            echo_error(e.args[0])
+
     if list_databases:
         cur, headers, status = pgcli.pgexecute.full_databases()
 
@@ -1830,6 +1888,7 @@ def format_output(title, cur, headers, status, settings, explain_mode=False):
         "missing_value": settings.missingval,
         "integer_format": settings.dcmlfmt,
         "float_format": settings.floatfmt,
+        "column_date_formats": settings.column_date_formats,
         "preprocessors": (format_numbers, format_arrays),
         "disable_numparse": True,
         "preserve_whitespace": True,
@@ -1839,6 +1898,9 @@ def format_output(title, cur, headers, status, settings, explain_mode=False):
     if not settings.floatfmt:
         output_kwargs["preprocessors"] = (align_decimals,)
 
+    if settings.column_date_formats:
+        output_kwargs["preprocessors"] += (format_timestamps,)
+
     if table_format == "csv":
         # The default CSV dialect is "excel" which is not handling newline values correctly
         # Nevertheless, we want to keep on using "excel" on Windows since it uses '\r\n'
